package de.cliffbreak.varo.managers;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;

import org.bukkit.Bukkit;
import org.bukkit.ChatColor;
import org.bukkit.entity.Player;
import org.bukkit.scoreboard.Scoreboard;
import org.bukkit.scoreboard.Team;
import org.json.simple.JSONArray;
import org.json.simple.JSONObject;
import org.json.simple.JSONValue;
import org.json.simple.parser.ParseException;

import de.cliffbreak.varo.Varo;
import de.cliffbreak.varo.uitls.JSONConfig;
import de.cliffbreak.varo.uitls.WebUtils;

public class TeamManager {

    private final Varo plugin;
    private final JSONConfig config;
    private final Scoreboard scoreboard;
    private final ArrayList<Team> teams;
    private static final String TEAM_PREFIX = "~";

    public TeamManager(final Varo plugin) {
        this.plugin = plugin;
        this.config = new JSONConfig(new File("plugins/CliffbreakVaro", "teams.json"), getDefaults());
        this.config.save();

        this.scoreboard = Bukkit.getScoreboardManager().getMainScoreboard();
        this.teams = new ArrayList<Team>();

        this.init();
    }

    private void init() {
        final JSONArray jsonTeams = config.getArray("Teams");
        for (final Object jsonRawTeam : jsonTeams) {
            final JSONObject jsonTeam = (JSONObject) jsonRawTeam;
            final Team team, afkTeam;
            if (jsonTeam.get("Name") == null) {
                continue;
            }
            if (this.scoreboard.getTeam((String) jsonTeam.get("Name")) == null) {
                team = this.scoreboard.registerNewTeam((String) jsonTeam.get("Name"));
            } else {
                team = this.scoreboard.getTeam((String) jsonTeam.get("Name"));
            }
            if (this.scoreboard.getTeam(TEAM_PREFIX + jsonTeam.get("Name")) == null) {
                afkTeam = this.scoreboard.registerNewTeam(TEAM_PREFIX + jsonTeam.get("Name"));
            } else {
                afkTeam = this.scoreboard.getTeam(TEAM_PREFIX + jsonTeam.get("Name"));
            }
            this.teams.add(team);
            this.teams.add(afkTeam);
            team.setPrefix(ChatColor.valueOf((String) jsonTeam.get("Color")) + "§l[" + jsonTeam.get("Prefix") + "] ");
            afkTeam.setPrefix("§8§l[AFK] ");
            team.setColor(ChatColor.WHITE);
            afkTeam.setColor(ChatColor.valueOf((String) jsonTeam.get("Color")));
            for (final String entry : team.getEntries()) {
                team.removeEntry(entry);
            }
            for (final String entry : afkTeam.getEntries()) {
                team.removeEntry(entry);
            }
            for (final Object uuid : (JSONArray) jsonTeam.get("Players")) {
                try {
                    final String result = WebUtils
                            .GET("https://sessionserver.mojang.com/session/minecraft/profile/" + uuid);
                    if (result.isEmpty()) {
                        continue;
                    }
                    final JSONObject profile = (JSONObject) JSONValue.parseWithException(result);
                    team.addEntry((String) profile.get("name"));

                } catch (IOException | ParseException e) {
                    this.plugin.getLogger().info(
                            "§c§lFehler:§r§c beim Verbinden mit der Mojang-API ist ein Fehler aufgetreten. Eventuell sind die Mojang Server down.");
                }
            }
        }
    }

    public Team getTeamByPlayer(final String player) {
        for (final Team team : this.teams) {
            if (team.hasEntry(player)) {
                return team;
            }
        }
        return null;
    }

    public void setAFK(final String player) {
        final Team oldTeam = getTeamByPlayer(player);
        for (final Team team : this.teams) {
            if (team.getName().equals(TEAM_PREFIX + oldTeam.getName())) {
                team.addEntry(player);
            }
        }
    }

    public void setTeam(final Player player) {
        final JSONArray jsonTeams = config.getArray("Teams");
        for (final Object jsonRawTeam : jsonTeams) {
            final JSONObject jsonTeam = (JSONObject) jsonRawTeam;
            if (((JSONArray) jsonTeam.get("Players")).contains(player.getUniqueId().toString().replace("-", ""))) {
                for (final Team team : this.teams) {
                    if (team.getName().equals(jsonTeam.get("Name").toString())) {
                        team.addEntry(player.getName());
                    }
                }
            }
        }
    }

    @SuppressWarnings("unchecked")
    private HashMap<String, Object> getDefaults() {
        final HashMap<String, Object> defaults = new HashMap<String, Object>();

        final JSONObject team1 = new JSONObject();
        team1.put("Name", "Demo-Team");
        team1.put("Color", ChatColor.RED);
        team1.put("Prefix", "DMO");
        final JSONArray team1Players = new JSONArray();
        team1Players.add("a6019ccffe694bf780bcc3f3afa265dd");
        team1Players.add("069a79f444e94726a5befca90e38aaf5");
        team1.put("Players", team1Players);

        final JSONObject team2 = new JSONObject();
        team2.put("Name", "Demo-Team 2");
        team2.put("Color", ChatColor.GREEN);
        team2.put("Prefix", "DM2");
        final JSONArray team2Players = new JSONArray();
        team2Players.add("dff082fcd52d4ad2b2d8d8202aea53ec");
        team2.put("Players", team2Players);

        final JSONArray teams = new JSONArray();
        teams.add(team1);
        teams.add(team2);
        defaults.put("Teams", teams);
        return defaults;
    }
}